/**
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    RobotAPI::GraspingWithTorques
* @author     Markus Przybylski
* @date       2014 Markus Przybylski
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef ARMARX_COMPONENT_GRASPINT_WITH_TORQUES_H
#define ARMARX_COMPONENT_GRASPINT_WITH_TORQUES_H

#include <Core/statechart/Statechart.h>


namespace armarx
{
    // ****************************************************************
    // Events
    // ****************************************************************

    //only for first tests
    DEFINEEVENT(EvPreshapeTimeout_ToCloseWithTorques)
    DEFINEEVENT(EvPreshapeTimeout_ToCloseWithJointAngles)
    DEFINEEVENT(EvMinExecutionTimeout)
    DEFINEEVENT(EvGraspWithJointAnglesTimeout)
    DEFINEEVENT(EvGraspWithTorquesTimeout)
    DEFINEEVENT(EvAllJointVelocitiesLow)

    // ****************************************************************
    // Definition of StatechartGraspingWithTorques
    // ****************************************************************
    struct StatechartGraspingWithTorques :
        StateTemplate<StatechartGraspingWithTorques>
    {
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();

    };



    // ****************************************************************
    // Definition of StatePreshape
    // ****************************************************************
    /**
     * StatePreshape: Move the fingers to a preshape configuration, then wait a little.
     */

    struct StatePreshape :
        StateTemplate<StatePreshape>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        StateUtility::ActionEventIdentifier condPreshapeTimeout;
    };

    // ****************************************************************
    // Definition of StateCloseHandWithTorques
    // ****************************************************************

    /**
     * StateCloseHandWithTorques: Put torques on the hand's joints.
     */
    struct StateCloseHandWithTorques :
        StateTemplate<StateCloseHandWithTorques>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        //Make sure that the joints have time to start moving before we check
        //low joint velocities as a termination criterion.
        StateUtility::ActionEventIdentifier condMinimumExecutionTimeout;
    };

    // ****************************************************************
    // Definition of StateCloseHandWithJointAngles
    // ****************************************************************

    /**
     * StateCloseHandWithJointAngles: Close the hand using a target joint angle configuration.
     */
    struct StateCloseHandWithJointAngles :
        StateTemplate<StateCloseHandWithJointAngles>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        //Make sure that the joints have time to start moving before we check
        //low joint velocities as a termination criterion.
        StateUtility::ActionEventIdentifier condGraspWithJointAnglesTimeout;
    };


    // ****************************************************************
    // Definition of StateInstallTerminateConditions
    // ****************************************************************
    /**
     * StateInstallTerminateConditions:
     */

    struct StateInstallTerminateConditions :
        StateTemplate<StateInstallTerminateConditions>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        ConditionIdentifier condAllJointVelocitiesLow;
        StateUtility::ActionEventIdentifier condGraspTimeout;
    };


}

#endif
