/**
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    RobotAPI::GraspingWithTorques
* @author     Markus Przybylski
* @date       2014 Markus Przybylski
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef ARMARX_COMPONENT_MOVE_PLATFORM_H
#define ARMARX_COMPONENT_MOVE_PLATFORM_H

#include <Core/statechart/Statechart.h>
#include <RobotAPI/statecharts/MovePlatform/PlatformContext.h>

namespace armarx
{
    class MovePlatformStateChart : public RemoteStateOfferer<PlatformContext>
    {
    public:
        void onInitRemoteStateOfferer();
        void onConnectRemoteStateOfferer();
        std::string getStateOffererName() const { return "MovePlatformStateChart"; }
    };

    // ****************************************************************
    // Events
    // ****************************************************************

//    DEFINEEVENT(EvSuccess)
    DEFINEEVENT(EvTimeoutTargetUnreachable)
    DEFINEEVENT(EvNoTargets)
    DEFINEEVENT(EvTargetReached)

    // ****************************************************************
    // Definition of StatechartMovePlatform
    // ****************************************************************
    struct StatechartMovePlatform :
        StateTemplate<StatechartMovePlatform>
    {
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();

    };

    // ****************************************************************
    // Definition of StateMoveToNext
    // ****************************************************************
    /**
     * StateMoveToNext: Move to next target in list
     */

    struct StateMoveToNext :
        StateTemplate<StateMoveToNext>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        ConditionIdentifier condTargetReached;
        StateUtility::ActionEventIdentifier timeoutMoveToNext;
    };

}

#endif
