/**
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    RobotAPI::OpenHand
* @author     Markus Przybylski
* @date       2014 Markus Przybylski
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef ARMARX_COMPONENT_OPEN_HAND_H
#define ARMARX_COMPONENT_OPEN_HAND_H

#include <Core/statechart/Statechart.h>


namespace armarx
{
    // ****************************************************************
    // Events
    // ****************************************************************

    //only for first tests
    DEFINEEVENT(EvOpenHandSuccessTimeout)
    DEFINEEVENT(EvOpenHandDummyFailureTimeout)


    // ****************************************************************
    // Definition of StatechartOpenHand
    // ****************************************************************
    struct StatechartOpenHand :
        StateTemplate<StatechartOpenHand>
    {
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();

    };



    // ****************************************************************
    // Definition of StateOpenHand
    // ****************************************************************
    /**
     * StateOpenHand: Move the fingers to a open configuration, then wait a little.
     */

    struct StateOpenHand :
        StateTemplate<StateOpenHand>
    {
        void defineParameters();
        void onEnter();
        void onExit();

        StateUtility::ActionEventIdentifier condOpenHandSuccessTimeout;
        StateUtility::ActionEventIdentifier condOpenHandDummyFailureTimeout;
    };









}

#endif
