#include "ForceTorqueObserver.h"

#include <Core/observers/checks/ConditionCheckUpdated.h>
#include <Core/observers/checks/ConditionCheckEquals.h>
#include <Core/observers/checks/ConditionCheckInRange.h>
#include <Core/observers/checks/ConditionCheckLarger.h>
#include <Core/observers/checks/ConditionCheckSmaller.h>
#include <Core/robotstate/remote/checks/ConditionCheckMagnitudeChecks.h>

#include <Core/robotstate/remote/RobotStateObjectFactories.h>

#define RAWFORCE "_rawforcevectors"
#define OFFSETFORCE "_forceswithoffsetvectors"
#define FILTEREDFORCE "_filteredforcesvectors"
#define RAWTORQUE "_rawtorquevectors"
#define OFFSETTORQUE "_torqueswithoffsetvectors"
#define FORCETORQUEVECTORS "_forceTorqueVectors"


using namespace armarx;

ForceTorqueObserver::ForceTorqueObserver()
{
}

void ForceTorqueObserver::setTopicName(std::string topicName)
{
    this->topicName = topicName;
}

void ForceTorqueObserver::onInitObserver()
{
    if(topicName.empty())
        usingTopic(getProperty<std::string>("ForceTorqueTopicName").getValue());
    else
        usingTopic(topicName);

    // register all checks
    offerConditionCheck("updated", new ConditionCheckUpdated());
    offerConditionCheck("larger", new ConditionCheckLarger());
    offerConditionCheck("equals", new ConditionCheckEquals());
    offerConditionCheck("smaller", new ConditionCheckSmaller());
//    offerConditionCheck("magnitudeinrange", new ConditionCheckInRange());
    offerConditionCheck("magnitudelarger", new ConditionCheckMagnitudeLarger());
//    offerConditionCheck("magnitudesmaller", new ConditionCheckSmaller());

    // register all channels
//    offerChannel(FORCETORQUEVECTORS,"Force and Torque vectors on specific parts of the robot.");

//    offerChannel(RAWFORCE,"Forces on specific parts of the robot.");
//    offerChannel(OFFSETFORCE,"Force Torques of specific parts of the robot with an offset.");
//    offerChannel(FILTEREDFORCE,"Gaussian filtered force torques of specific parts of the robot.");

//    offerChannel(RAWTORQUE,"Torques on specific parts of the robot.");

}

void ForceTorqueObserver::onConnectObserver()
{


}

//void ForceTorqueObserver::reportForceRaw(const FramedVector3Map &newForces, const Ice::Current &)
//{

//}

//void ForceTorqueObserver::reportForceWithOffset(const FramedVector3Map &newForces, const Ice::Current &)
//{
//    FramedVector3Map::const_iterator it = newForces.begin();
//    for(; it != newForces.end(); it++)
//    {

//        FramedVector3Ptr vec = FramedVector3Ptr::dynamicCast(it->second);
//        if(!existsDataField(OFFSETFORCE, it->first))
//            offerDataFieldWithDefault(OFFSETFORCE, it->first, Variant(it->second), "3D vector for Force Torques of " + it->first);
//        else
//            setDataField(OFFSETFORCE, it->first, Variant(it->second));


//        if(!existsChannel(it->first))
//        {
//            offerChannel(it->first,"Force on " + it->first);
//            offerDataFieldWithDefault(it->first,"force_x", Variant(vec->x), "Force on X axis");
//            offerDataFieldWithDefault(it->first,"force_y", Variant(vec->y), "Force on Y axis");
//            offerDataFieldWithDefault(it->first,"force_z", Variant(vec->z), "Force on Z axis");
//        }
//        else
//        {
//            setDataField(it->first,"force_x", Variant(vec->x));
//            setDataField(it->first,"force_y", Variant(vec->y));
//            setDataField(it->first,"force_z", Variant(vec->z));
//        }

//    }
//}



void ForceTorqueObserver::reportSensorValues(const std::string &type, const FramedVector3Map &newValues, const Ice::Current &)
{
    ScopedLock lock(dataMutex);
    FramedVector3Map::const_iterator it = newValues.begin();
    if(!existsChannel(type))
        offerChannel(type, "Force and Torque vectors on specific parts of the robot.");
    for(; it != newValues.end(); it++)
    {

        FramedVector3Ptr vec = FramedVector3Ptr::dynamicCast(it->second);
        std::string identifier = it->first + "_" + type + "_in_frame_" + vec->frame;
        if(!existsDataField(type, identifier))
            offerDataFieldWithDefault(type, identifier, Variant(it->second), "3D vector for Force Torques of " + it->first);
        else
            setDataField(type, identifier, Variant(it->second));

        if(!existsChannel(identifier))
        {
            offerChannel(identifier,type + " on " + it->first);
            offerDataFieldWithDefault(identifier,type + "_x", Variant(vec->x), type + " on X axis");
            offerDataFieldWithDefault(identifier,type + "_y", Variant(vec->y), type + " on Y axis");
            offerDataFieldWithDefault(identifier,type + "_z", Variant(vec->z), type + " on Z axis");
            offerDataFieldWithDefault(identifier,type + "_frame", Variant(vec->frame), "Frame of " + type);
        }
        else
        {
            setDataField(identifier,type + "_x", Variant(vec->x));
            setDataField(identifier,type + "_y", Variant(vec->y));
            setDataField(identifier,type + "_z", Variant(vec->z));
            setDataField(identifier,type + "_frame", Variant(vec->frame));
        }

    }
}

PropertyDefinitionsPtr ForceTorqueObserver::createPropertyDefinitions()
{
    return PropertyDefinitionsPtr(new ForceTorqueObserverPropertyDefinitions(
                                           getConfigIdentifier()));
}
